@@ -1,10 +1,10 @@
 /*
-  Copyright (C) 2006 Werner Dittmann
+  Copyright (C) 2006-2007 Werner Dittmann
 
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /*
@@ -25,40 +24,52 @@
 
 ZrtpPacketDHPart::ZrtpPacketDHPart(SupportedPubKeys pkt) {
     DEBUGOUT((fprintf(stdout, "Creating DHPart packet without data\n")));
+#if 0
+    int length = sizeof(DHPart_t) + sizeof(zrtpPacketHeader_t) + CRC_SIZE + ((pkt == Dh3072) ? 384 : 512);
 
-    int length = sizeof(zrtpPacketHeader_t) + sizeof(DHPart_t);
-
-    length += ((pkt == Dh3072) ? 384 : 512); // length according to DH type
-    allocated = malloc(length);
+    void* allocated = &data;
     memset(allocated, 0, length);
 
-    if (allocated == NULL) {
-	// TODO error handling
-    }
-
     pktype = pkt;
 
     zrtpHeader = (zrtpPacketHeader_t *)&((DHPartPacket_t *)allocated)->hdr;	// the standard header
-    pv = ((uint8_t *)allocated) + sizeof(zrtpPacketHeader_t); 		// point to the public key value
-    DHPartHeader = (DHPart_t *)(((char *)allocated)+sizeof(zrtpPacketHeader_t)+((pkt == Dh3072) ? 384 : 512));
+    pv = ((uint8_t *)allocated) + sizeof(zrtpPacketHeader_t);    // point to the public key value
+    DHPartHeader = (DHPart_t *)(pv + ((pkt == Dh3072) ? 384 : 512));
 
     setZrtpId();
-    setLength(DHPART_LENGTH + MESSAGE_LENGTH + ((pkt == Dh3072) ? 96 : 128));
+    // Subtract one to exclude the CRC word from length in ZRTP message 
+    setLength((length / 4) - 1);
+    // setLength(DHPART_LENGTH + MESSAGE_LENGTH + ((pkt == Dh3072) ? 96 : 128));
+#else
+    int length = sizeof(DHPartPacket_t) + CRC_SIZE + ((pkt == Dh3072) ? 384 : 512);
+
+    void* allocated = &data;
+    memset(allocated, 0, length);
+
+    pktype = pkt;
+
+    zrtpHeader = (zrtpPacketHeader_t *)&((DHPartPacket_t *)allocated)->hdr;	// the standard header
+    DHPartHeader = (DHPart_t *)&((DHPartPacket_t *)allocated)->dhPart;
+    pv = ((uint8_t *)allocated) + sizeof(DHPartPacket_t);    // point to the public key value
+
+    setZrtpId();
+    // Subtract one to exclude the CRC word from length in ZRTP message 
+    setLength((length / 4) - 1);
+#endif
 }
 
 ZrtpPacketDHPart::ZrtpPacketDHPart(uint8_t *data) {
     DEBUGOUT((fprintf(stdout, "Creating DHPart packet from data\n")));
-
-    allocated = NULL;
+#if 0
     zrtpHeader = (zrtpPacketHeader_t *)&((DHPartPacket_t *)data)->hdr;	// the standard header
 
     int16_t len = getLength();
     DEBUGOUT((fprintf(stdout, "DHPart length: %d\n", len)));
     SupportedPubKeys pkt;
-    if (len == 108) {
+    if (len == 109) {
 	pkt = Dh3072;
     }
-    else if (len == 140) {
+    else if (len == 141) {
 	pkt = Dh4096;
     }
     else {
@@ -68,14 +79,30 @@ ZrtpPacketDHPart::ZrtpPacketDHPart(uint8_t *data) {
     }
     pv = data + sizeof(zrtpPacketHeader_t);
     DHPartHeader = (DHPart_t *)(data + sizeof(zrtpPacketHeader_t) + ((pkt == Dh3072) ? 384 : 512));
-
     pktype = pkt;
+#else
+    zrtpHeader = (zrtpPacketHeader_t *)&((DHPartPacket_t *)data)->hdr;	// the standard header
+    DHPartHeader = (DHPart_t *)&((DHPartPacket_t *)data)->dhPart;
+
+    int16_t len = getLength();
+    DEBUGOUT((fprintf(stdout, "DHPart length: %d\n", len)));
+    SupportedPubKeys pkt;
+    if (len == 109) {
+	pkt = Dh3072;
+    }
+    else if (len == 141) {
+	pkt = Dh4096;
+    }
+    else {
+	fprintf(stderr, "Wrong DHPart length: %d\n", len);
+	pv = NULL;
+	return;
+    }
+    pv = data + sizeof(DHPartPacket_t);    // point to the public key value
+    pktype = pkt;
+#endif
 }
 
 ZrtpPacketDHPart::~ZrtpPacketDHPart() {
     DEBUGOUT((fprintf(stdout, "Deleting DHPart packet: alloc: %x\n", allocated)));
-
-    if (allocated != NULL) {
-	free(allocated);
-    }
 }
