@@ -1,10 +1,10 @@
 /*
-  Copyright (C) 2006 Werner Dittmann
+  Copyright (C) 2006-2007 Werner Dittmann
 
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /**
@@ -39,7 +38,9 @@ state_t states[numberOfStates] = {
     {WaitConfirm1, &ZrtpStateClass::evWaitConfirm1 },
     {WaitConfirm2, &ZrtpStateClass::evWaitConfirm2 },
     {WaitConfAck,  &ZrtpStateClass::evWaitConfAck },
-    {SecureState,  &ZrtpStateClass::evSecureState }
+    {WaitClearAck, &ZrtpStateClass::evWaitClearAck },
+    {SecureState,  &ZrtpStateClass::evSecureState },
+    {WaitErrorAck, &ZrtpStateClass::evWaitErrorAck }
 };
 
 static char* sendErrorText = "Cannot send data via RTP - connection or peer down?";
@@ -73,12 +74,37 @@ ZrtpStateClass::~ZrtpStateClass(void) {
 int32_t ZrtpStateClass::processEvent(Event_t *ev) {
 
     /*
-     * Ignore any events if we are not really started yet.
+     * Ignore any events except Initial if we are not really 
+     * started yet.
      */
-    if (inState(Initial) && ev->type != ZrtpInitial) {
+    if (inState(Initial) && !(ev->type == ZrtpInitial)) {
 	return (Done);
     }
     event = ev;
+    char *msg, first, last;
+    uint8_t *pkt;
+    if (event->type == ZrtpPacket) {
+	pkt = event->data.packet;
+	msg = (char *)pkt + 4;
+	first = tolower(*msg);
+	last = tolower(*(msg+4));
+
+        // Check if this is an Error packet.
+	if (first == 'e' && last =='r') {
+            /*
+             * Process a received Error packet.
+             *
+             * Use callback method to prepare and get an ErrorAck packet. Send this
+             * to our peer and reset to protocol engine to Initial state.
+             */
+            ZrtpPacketError* epkt = new ZrtpPacketError(pkt);
+            ZrtpPacketErrorAck* eapkt =  parent->prepareErrorAck(epkt);
+            delete epkt;
+            parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(eapkt));
+            nextState(Initial);
+            return (Done);
+        }
+    }
     return engine->processEvent(*this);
 }
 
@@ -86,23 +112,40 @@ int32_t ZrtpStateClass::processEvent(Event_t *ev) {
 int32_t ZrtpStateClass::evInitial(void) {
     DEBUGOUT((cout << "Checking for match in Initial.\n"));
 
-    ZrtpPacketHello *hello = parent->prepareHello();
+    if (event->type == ZrtpInitial) {
+	ZrtpPacketHello *hello = parent->prepareHello();
 
-    // remember packet for easy resend in case timer triggers
-    sentPacket = static_cast<ZrtpPacketBase *>(hello);
+	// remember packet for easy resend in case timer triggers
+	sentPacket = static_cast<ZrtpPacketBase *>(hello);
 
-    if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(hello)) || (startTimer(&T1) <= 0)) {
-	nextState(Initial);
-	parent->sendInfo(Error, sendErrorText);
-	return(Fail);
+	if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(hello)) || (startTimer(&T1) <= 0)) {
+	    nextState(Initial);
+	    parent->sendInfo(Error, sendErrorText);
+	    return(Fail);
+	}
+	nextState(Detect);
     }
-    nextState(Detect);
-    return(Done);
+    return (Done);
 }
 
-/*
- * When entering this transition function
- * - sentPacket contains Hello, Hello timer active
+/**
+ * Detect state.
+ *
+ * When in this state the protocol engine sent an Hello packet to the peer.
+ * When entering this state transition function then:
+ * - Assume Initiator mode, mode may change later on peer reaction
+ * - Instance variable sentPacket contains the sent Hello packet
+ * - Hello timer T1 is active
+ *
+ * Possible events in this state are: 
+ * timeout for sent Hello packet - causes a resend check an possible resend of
+ * Hello packet
+ * received a Commit - stop active timer, prepare a DHPart1 packet, switch to 
+ * Resonder mode
+ * received a HelloAck - stop active timer, prepare and send Hello packet.
+ * received a Hello - stop active timer, prepare and send Commit packet.
+ * 
+ * Refer to state event diagram to see which are valid next states.
  */
 int32_t ZrtpStateClass::evDetect(void) {
 
@@ -110,7 +153,12 @@ int32_t ZrtpStateClass::evDetect(void) {
 
     char *msg, first, last;
     uint8_t *pkt;
+    uint32_t errorCode = 0;
 
+    /*
+     * First check the general event type, then discrimnate
+     * the real event.
+     */
     if (event->type == ZrtpPacket) {
 	pkt = event->data.packet;
 	msg = (char *)pkt + 4;
@@ -120,22 +168,30 @@ int32_t ZrtpStateClass::evDetect(void) {
 
 	/*
 	 * Commit:
-	 * - go the responder path
-	 * - send our DHPart1
-	 * - switch to state WaitDHPart2, wait for peer's DHPart2
+         * The peer answers with Commit to our Hello, thus switch to 
+         * responder mode.
+	 * - prepare and send our DHPart1
+	 * - switch to state WaitDHPart2 and wait for peer's DHPart2
 	 * - don't start timer, we are responder
 	 */
 	if (first == 'c') {
+	    ZrtpPacketCommit *cpkt = new ZrtpPacketCommit(pkt);
+	    // Only if ZRTP CRC is ok then accept packet and process event
 	    cancelTimer();	// stop Hello timer processing, don't delete a Hello packet
 	    sentPacket = NULL;
 
-	    ZrtpPacketCommit *cpkt = new ZrtpPacketCommit(pkt);
-	    ZrtpPacketDHPart* dhPart1 = parent->prepareDHPart1(cpkt);
+	    // TODO: check for multistream request
+	    ZrtpPacketDHPart* dhPart1 = parent->prepareDHPart1(cpkt, &errorCode);
 	    delete cpkt;
 
+	    // Something went wrong during processing of the commit packet
+	    if (dhPart1 == NULL) {
+                sendErrorPacket(errorCode);
+		return (Done);
+	    }
 	    nextState(WaitDHPart2);
 
-	    if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(dhPart1))) {
+	    if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(dhPart1))) {
 		delete dhPart1;
 		nextState(Initial);
 		parent->sendInfo(Error, sendErrorText);
@@ -148,10 +204,10 @@ int32_t ZrtpStateClass::evDetect(void) {
 	/*
 	 * HelloAck:
 	 * - stop resending Hello,
-	 * - switch to state AckDetected, wait for peer's Hello
+	 * - switch to state AckDetected, wait for peer's Hello (F3)
 	 */
 	if (first == 'h' && last =='k') {
-	    cancelTimer();	// stop Hello timer processing, don't delete this Hello packet
+	    cancelTimer();	// stop Hello timer processing, don't delete my Hello packet
 	    sentPacket = NULL;
 	    nextState(AckDetected);
 	    return (Done);
@@ -160,24 +216,28 @@ int32_t ZrtpStateClass::evDetect(void) {
 	 * Hello:
 	 * - stop Hello timer
 	 * - prepare and send my Commit,
-	 * - switch state to CommitSent, start Commit timer
+	 * - switch state to CommitSent, start Commit timer, assume Initiator
 	 */
 	if (first == 'h' && last ==' ') {
+	    ZrtpPacketHello *hpkt = new ZrtpPacketHello(pkt);
 	    cancelTimer();
-
-            parent->sendPacketRTP(sentPacket);    // just resend Hello in case peer missed ours
+            parent->sendPacketZRTP(sentPacket);    // just resend Hello in case peer missed ours
 	    sentPacket = NULL;
 
-	    ZrtpPacketHello *hpkt = new ZrtpPacketHello(pkt);
-	    ZrtpPacketCommit* commit = parent->prepareCommit(hpkt);
+	    ZrtpPacketCommit* commit = parent->prepareCommit(hpkt,  &errorCode);
 	    delete hpkt;
 
+	    // Something went wrong during processing of the Hello packet
+	    if (commit == NULL) {
+                sendErrorPacket(errorCode);
+                return (Done);
+	    }
 	    nextState(CommitSent);
 
             // remember packet for easy resend in case timer triggers
             // Timer trigger received in new state CommitSend
             sentPacket = static_cast<ZrtpPacketBase *>(commit);
-            if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(commit)) || (startTimer(&T2) <= 0)) {
+            if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(commit)) || (startTimer(&T2) <= 0)) {
 		delete sentPacket;
 		sentPacket = NULL;
 		nextState(Initial);
@@ -191,11 +251,11 @@ int32_t ZrtpStateClass::evDetect(void) {
     // Timer event triggered - this is Timer T1 to resend Hello
     else if (event->type == Timer) {
 	if (sentPacket != NULL) {
-            if ((nextTimer(&T1)) <= 0 || !parent->sendPacketRTP(sentPacket)) {
+            if ((nextTimer(&T1)) <= 0 || !parent->sendPacketZRTP(sentPacket)) {
                 parent->zrtpNotSuppOther();
                 sentPacket = NULL;
-                // Switch state "Detect" tobe prepared to receive Hello from
-                // other side any time later
+                // Stay in state Detect to be prepared get an hello from
+                // other peer any time later
                 nextState(Detect);
                 return (Fail);
             }
@@ -203,12 +263,7 @@ int32_t ZrtpStateClass::evDetect(void) {
         return (Done);
     }
     else {      // unknown Event type for this state
-        if (event->type == ZrtpGoClear) {
-            parent->sendInfo(Error, goClearReceived);
-        }
-        else {
-            parent->sendInfo(Error, internalProtocolError);
-        }
+        parent->sendInfo(Error, internalProtocolError);
 	sentPacket = NULL;
 	nextState(Initial);
         return (Fail);
@@ -225,6 +280,7 @@ int32_t ZrtpStateClass::evAckDetected(void) {
 
     char *msg, first, last;
     uint8_t *pkt;
+    uint32_t errorCode = 0;
 
     if (event->type == ZrtpPacket) {
 	pkt = event->data.packet;
@@ -234,17 +290,19 @@ int32_t ZrtpStateClass::evAckDetected(void) {
 	last = tolower(*(msg+7));
 	/*
 	 * Hello:
-	 * - Acknowledge the Hello
+	 * - Acknowledge my peers Hello, sending HelloACK (F4)
 	 * - switch to state WaitCommit, wait for peer's Commit
+         * - we are going to be in the Responder role
 	 */
 	if (first == 'h') {
-            ZrtpPacketHello *hpkt = new ZrtpPacketHello(pkt);
-            ZrtpPacketHelloAck *helloAck = parent->prepareHelloAck(hpkt);
-            delete hpkt;
+// NOTE: may be useful if protocol state engine changes (preshared mode)
+//            ZrtpPacketHello *hpkt = new ZrtpPacketHello(pkt);
+            ZrtpPacketHelloAck *helloAck = parent->prepareHelloAck();
+//            delete hpkt;
 
 	    nextState(WaitCommit);
 
-	    if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(helloAck))) {
+	    if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(helloAck))) {
 		nextState(Initial);
 		sentPacket = NULL;
 		parent->sendInfo(Error, sendErrorText);
@@ -255,8 +313,8 @@ int32_t ZrtpStateClass::evAckDetected(void) {
 	    return (Done);
 	}
        /*
-        * Commit:  (Actually illegal here, but handle it because of problem in Zfone)
-        * - Act as Responder
+        * Commit:
+        * - act as Responder
         * - send our DHPart1
         * - switch to state WaitDHPart2, wait for peer's DHPart2
         * - don't start timer, we are responder
@@ -264,12 +322,17 @@ int32_t ZrtpStateClass::evAckDetected(void) {
         if (first == 'c') {
 
             ZrtpPacketCommit *cpkt = new ZrtpPacketCommit(pkt);
-            ZrtpPacketDHPart* dhPart1 = parent->prepareDHPart1(cpkt);
+            ZrtpPacketDHPart* dhPart1 = parent->prepareDHPart1(cpkt, &errorCode);
             delete cpkt;
 
+            // Something went wrong during processing of the Commit packet
+            if (dhPart1 == NULL) {
+                sendErrorPacket(errorCode);
+                return (Done);
+            }
             nextState(WaitDHPart2);
 
-            if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(dhPart1))) {
+            if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(dhPart1))) {
                 delete dhPart1;
                 sentPacket = NULL;
                 nextState(Initial);
@@ -283,12 +346,7 @@ int32_t ZrtpStateClass::evAckDetected(void) {
         return (Done);      // unknown packet for this state - Just ignore it
     }
     else {
-        if (event->type == ZrtpGoClear) {
-            parent->sendInfo(Error, goClearReceived);
-        }
-        else {
-            parent->sendInfo(Error, internalProtocolError);
-        }
+        parent->sendInfo(Error, internalProtocolError);
 	sentPacket = NULL;
 	nextState(Initial);
         return (Fail);
@@ -297,6 +355,7 @@ int32_t ZrtpStateClass::evAckDetected(void) {
 
 /*
  * When entering this transition function
+ * - Responder mode
  * - sentPacket contains a HelloAck packet
  */
 int32_t ZrtpStateClass::evWaitCommit(void) {
@@ -305,6 +364,7 @@ int32_t ZrtpStateClass::evWaitCommit(void) {
 
     char *msg, first, last;
     uint8_t *pkt;
+    uint32_t errorCode = 0;
 
     if (event->type == ZrtpPacket) {
 	pkt = event->data.packet;
@@ -319,7 +379,9 @@ int32_t ZrtpStateClass::evWaitCommit(void) {
 	 * - stay in WaitCommit
 	 */
 	if (first == 'h') {
-	    if (!parent->sendPacketRTP(sentPacket)) {
+            ZrtpPacketHello *hpkt = new ZrtpPacketHello(pkt);
+	    delete hpkt;
+	    if (!parent->sendPacketZRTP(sentPacket)) {
 		nextState(Initial);
 		sentPacket = NULL;    // don't delete HelloAck, it's preconfigured
 		parent->sendInfo(Error, sendErrorText);
@@ -335,14 +397,19 @@ int32_t ZrtpStateClass::evWaitCommit(void) {
 	 * - don't start timer, we are responder
 	 */
 	if (first == 'c') {
-            sentPacket = NULL;
 	    ZrtpPacketCommit *cpkt = new ZrtpPacketCommit(pkt);
-	    ZrtpPacketDHPart* dhPart1 = parent->prepareDHPart1(cpkt);
+            sentPacket = NULL;
+	    ZrtpPacketDHPart* dhPart1 = parent->prepareDHPart1(cpkt, &errorCode);
 	    delete cpkt;
 
+            // Something went wrong during processing of the Commit packet
+            if (dhPart1 == NULL) {
+                sendErrorPacket(errorCode);
+                return (Done);
+            }
 	    nextState(WaitDHPart2);
 
-	    if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(dhPart1))){
+	    if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(dhPart1))){
 		delete dhPart1;
 		nextState(Initial);
 		parent->sendInfo(Error, sendErrorText);
@@ -354,12 +421,7 @@ int32_t ZrtpStateClass::evWaitCommit(void) {
         return (Done);      // unknown packet for this state - Just ignore it
     }
     else {
-        if (event->type == ZrtpGoClear) {
-            parent->sendInfo(Error, goClearReceived);
-        }
-        else {
-            parent->sendInfo(Error, internalProtocolError);
-        }
+        parent->sendInfo(Error, internalProtocolError);
         sentPacket = NULL;   // Don't delet sent packet - it's a fixed helloack
 	nextState(Initial);
         return (Fail);
@@ -368,6 +430,7 @@ int32_t ZrtpStateClass::evWaitCommit(void) {
 
 /*
  * When entering this transition function
+ * - assume Initiator mode, may change if we reveice a Commit here
  * - sentPacket contains Commit packet, Hello timer stopped, Commit timer active
  */
 
@@ -377,6 +440,7 @@ int32_t ZrtpStateClass::evCommitSent(void) {
 
     char *msg, first, last;
     uint8_t *pkt;
+    uint32_t errorCode = 0;
 
     if (event->type == ZrtpPacket) {
 	pkt = event->data.packet;
@@ -385,6 +449,15 @@ int32_t ZrtpStateClass::evCommitSent(void) {
 	first = tolower(*msg);
 	last = tolower(*(msg+7));
 
+        /*
+	 * HelloAck:
+	 * - late "helloAck", maybe  due to network latency, just ignore it
+	 * - no switch in state, leave timer as it is
+	 */
+	if (first == 'h' && last =='k') {
+	    return (Done);
+	}
+
 	/*
 	 * Commit:
          * We have a "Commit" clash. Resolve it.
@@ -400,6 +473,7 @@ int32_t ZrtpStateClass::evCommitSent(void) {
 	 */
 	if (first == 'c') {
 	    ZrtpPacketCommit *zpCo = new ZrtpPacketCommit(pkt);
+            sentPacket = NULL;
 	    cancelTimer();         // this cancels the Commit timer T2
 
 	    // if our hvi is less then peer's hvi - we are Responder and need
@@ -408,11 +482,16 @@ int32_t ZrtpStateClass::evCommitSent(void) {
 	    if (parent->compareHvi(zpCo) < 0) {
 		delete sentPacket;     // delete our sent Commit packet
 		sentPacket = NULL;
-		ZrtpPacketDHPart* dhPart1 = parent->prepareDHPart1(zpCo);
+		ZrtpPacketDHPart* dhPart1 = parent->prepareDHPart1(zpCo, &errorCode);
 
-		nextState(WaitDHPart2);
+                // Something went wrong during processing of the Commit packet
+                if (dhPart1 == NULL) {
+                    sendErrorPacket(errorCode);
+                    return (Done);
+                }
+                nextState(WaitDHPart2);
 
-		if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(dhPart1))){
+		if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(dhPart1))){
 		    delete dhPart1;
 		    delete zpCo;
 		    nextState(Initial);
@@ -442,27 +521,29 @@ int32_t ZrtpStateClass::evCommitSent(void) {
 	 * - Prepare and send DHPart2
 	 * - switch to WaitConfirm1
 	 * - start timer to resend DHPart2 if necessary, we are Initiator
-	 * - switch on Security for receiver, Confirm1 payload is encrypted
+	 * - switch on SRTP
 	 */
 	if (first == 'd') {
+	    ZrtpPacketDHPart* dpkt = new ZrtpPacketDHPart(pkt);
 	    cancelTimer();
 	    delete sentPacket;     // deletes the Commit packet
 	    sentPacket = NULL;
 
-	    ZrtpPacketDHPart* dpkt = new ZrtpPacketDHPart(pkt);
-	    ZrtpPacketDHPart* dhPart2 = parent->prepareDHPart2(dpkt);
+	    ZrtpPacketDHPart* dhPart2 = parent->prepareDHPart2(dpkt, &errorCode);
 	    delete dpkt;
 
+            // Something went wrong during processing of the DHPart1 packet
+            if (dhPart2 == NULL) {
+                sendErrorPacket(errorCode);
+                return (Done);
+            }
 	    nextState(WaitConfirm1);
-            // Next message should be Confirm1 and this already encrypted
-	    parent->srtpSecretsReady(ForReceiver);
 
             sentPacket = static_cast<ZrtpPacketBase *>(dhPart2);
-            if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(dhPart2)) || (startTimer(&T2) <= 0) ){
+            if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(dhPart2)) || (startTimer(&T2) <= 0) ){
 		delete sentPacket;
                 sentPacket = NULL;
 		nextState(Initial);
-		parent->srtpSecretsOff(ForReceiver);
 		parent->sendInfo(Error, sendErrorText);
 		return(Fail);
 	    }
@@ -473,7 +554,7 @@ int32_t ZrtpStateClass::evCommitSent(void) {
     // Timer event triggered, resend the Commit packet
     else if (event->type == Timer) {
 	if (sentPacket != NULL) {
-            if ((nextTimer(&T2) <= 0) || !parent->sendPacketRTP(sentPacket)) {
+            if ((nextTimer(&T2) <= 0) || !parent->sendPacketZRTP(sentPacket)) {
                 parent->sendInfo(Error, resendError);
                 delete sentPacket;
                 sentPacket = NULL;
@@ -484,12 +565,7 @@ int32_t ZrtpStateClass::evCommitSent(void) {
         return (Done);
     }
     else {      // unknown Event type for this state
-        if (event->type == ZrtpGoClear) {
-            parent->sendInfo(Error, goClearReceived);
-        }
-        else {
-            parent->sendInfo(Error, internalProtocolError);
-        }
+        parent->sendInfo(Error, internalProtocolError);
         delete sentPacket;
 	sentPacket = NULL;
 	nextState(Initial);
@@ -507,6 +583,7 @@ int32_t ZrtpStateClass::evWaitDHPart2(void) {
 
     char *msg, first, last;
     uint8_t *pkt;
+    uint32_t errorCode = 0;
 
     if (event->type == ZrtpPacket) {
 	pkt = event->data.packet;
@@ -521,7 +598,8 @@ int32_t ZrtpStateClass::evWaitDHPart2(void) {
 	 * - stay in state
 	 */
 	if (first == 'c') {
-	    if (!parent->sendPacketRTP(sentPacket)) {
+	    // ZrtpPacketCommit *zpCo = new ZrtpPacketCommit(pkt); TODO
+	    if (!parent->sendPacketZRTP(sentPacket)) {
 		delete sentPacket;
 		sentPacket = NULL;
 		nextState(Initial);
@@ -534,32 +612,26 @@ int32_t ZrtpStateClass::evWaitDHPart2(void) {
 	 * DHPart2:
 	 * - prepare Confirm1 packet
 	 * - switch to WaitConfirm2
-	 * - switch on security, send it via SRTP
+	 * - switch on SRTP
 	 * - No timer, we are responder
 	 */
 	if (first == 'd') {
+	    ZrtpPacketDHPart* dpkt = new ZrtpPacketDHPart(pkt);
 	    delete sentPacket;     // delete DHPart1 packet
 	    sentPacket = NULL;
 
-	    ZrtpPacketDHPart* dpkt = new ZrtpPacketDHPart(pkt);
-	    ZrtpPacketConfirm* confirm = parent->prepareConfirm1(dpkt);
+	    ZrtpPacketConfirm* confirm = parent->prepareConfirm1(dpkt, &errorCode);
 	    delete dpkt;
 
             if (confirm == NULL) {
-                nextState(Initial);
-                return(Fail);
+                sendErrorPacket(errorCode);
+                return (Done);
             }
 	    nextState(WaitConfirm2);
-            // switch on security for both directions, send confirm1 encrypted,
-            // wait for conf2Ack which is also encrypted
-            parent->srtpSecretsReady(ForSender);
-            parent->srtpSecretsReady(ForReceiver);
 
-	    if (!parent->sendPacketSRTP(static_cast<ZrtpPacketBase *>(confirm))){
+	    if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(confirm))){
 		delete confirm;
 		nextState(Initial);
-                parent->srtpSecretsOff(ForSender);
-                parent->srtpSecretsOff(ForReceiver);
                 parent->sendInfo(Error, sendErrorTextSrtp);
 		return(Fail);
 	    }
@@ -569,12 +641,7 @@ int32_t ZrtpStateClass::evWaitDHPart2(void) {
         return (Done);      // unknown packet for this state - Just ignore it
     }
     else {      // unknown Event type for this state
-        if (event->type == ZrtpGoClear) {
-            parent->sendInfo(Error, goClearReceived);
-        }
-        else {
-            parent->sendInfo(Error, internalProtocolError);
-        }
+        parent->sendInfo(Error, internalProtocolError);
         delete sentPacket;
         sentPacket = NULL;
         nextState(Initial);
@@ -584,8 +651,8 @@ int32_t ZrtpStateClass::evWaitDHPart2(void) {
 
 /*
  * When entering this transition function
+ * - Initiator mode
  * - sentPacket contains DHPart2 packet, DHPart2 timer active
- * - Receiver security switched on
  */
 int32_t ZrtpStateClass::evWaitConfirm1(void) {
 
@@ -593,6 +660,7 @@ int32_t ZrtpStateClass::evWaitConfirm1(void) {
 
     char *msg, first, last;
     uint8_t *pkt;
+    uint32_t errorCode = 0;
 
     if (event->type == ZrtpPacket) {
 	pkt = event->data.packet;
@@ -605,68 +673,63 @@ int32_t ZrtpStateClass::evWaitConfirm1(void) {
 	 * Confirm1:
 	 * - Switch off resending DHPart2
 	 * - prepare a Confirm2 packet
-	 * - switch on sender security and send via Confirm2 via SRTP,
 	 * - switch to state WaitConfAck
 	 * - set timer to monitor Confirm2 packet, we are initiator
 	 */
 	if (first == 'c' && last == '1') {
+	    ZrtpPacketConfirm* cpkt = new ZrtpPacketConfirm(pkt);
 	    cancelTimer();
 	    delete sentPacket;             // delete DHPart2 packet
 	    sentPacket = NULL;
 
-	    ZrtpPacketConfirm* cpkt = new ZrtpPacketConfirm(pkt, event->content);
-	    ZrtpPacketConfirm* confirm = parent->prepareConfirm2(cpkt);
+	    ZrtpPacketConfirm* confirm = parent->prepareConfirm2(cpkt, &errorCode);
 	    delete cpkt;
 
+            // Something went wrong during processing of the Confirm1 packet
+            if (confirm == NULL) {
+                sendErrorPacket(errorCode);
+                return (Done);
+            }
 	    nextState(WaitConfAck);
-	    parent->srtpSecretsReady(ForSender);	// let's try
 
             sentPacket = static_cast<ZrtpPacketBase *>(confirm);
-            if (!parent->sendPacketSRTP(static_cast<ZrtpPacketBase *>(confirm)) || (startTimer(&T2) <= 0)){
+            if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(confirm)) || (startTimer(&T2) <= 0)){
                 delete sentPacket;
                 sentPacket = NULL;
 		nextState(Initial);
-                parent->srtpSecretsOff(ForSender);
-                parent->srtpSecretsOff(ForReceiver);
 		parent->sendInfo(Error, sendErrorText);
 		return(Fail);
 	    }
-	    return (OkDismiss);
+	    return (Done);
 	}
         return (Done);      // unknown packet for this state - Just ignore it
     }
     else if (event->type == Timer) {
 	if (sentPacket != NULL) {
-            if ((nextTimer(&T2) <= 0) || !parent->sendPacketRTP(sentPacket)) {
+            if ((nextTimer(&T2) <= 0) || !parent->sendPacketZRTP(sentPacket)) {
                 parent->sendInfo(Error, resendError);
                 delete sentPacket;
                 sentPacket = NULL;
                 nextState(Initial);
-                parent->srtpSecretsOff(ForReceiver);
                 return (Fail);
             }
         }
         return (Done);
     }
     else {      // unknown Event type for this state
-        if (event->type == ZrtpGoClear) {
-            parent->sendInfo(Error, goClearReceived);
-        }
-        else {
-            parent->sendInfo(Error, internalProtocolError);
-        }
+        parent->sendInfo(Error, internalProtocolError);
         delete sentPacket;
         sentPacket = NULL;
 	nextState(Initial);
-	parent->srtpSecretsOff(ForReceiver);
         return (Fail);
     }
 }
 
 /*
  * When entering this transition function
+ * - Responder mode
  * - sentPacket contains Confirm1 packet, no timer active
- * - Sender security switched on
+ * - Security switched on
  */
 int32_t ZrtpStateClass::evWaitConfirm2(void) {
 
@@ -674,6 +737,7 @@ int32_t ZrtpStateClass::evWaitConfirm2(void) {
 
     char *msg, first, last;
     uint8_t *pkt;
+    uint32_t errorCode = 0;
 
     if (event->type == ZrtpPacket) {
 	pkt = event->data.packet;
@@ -688,7 +752,7 @@ int32_t ZrtpStateClass::evWaitConfirm2(void) {
 	 * - stay in state
 	 */
 	if (first == 'd') {
-	    if (!parent->sendPacketSRTP(sentPacket)) {
+	    if (!parent->sendPacketZRTP(sentPacket)) {
 		delete sentPacket;
 		sentPacket = NULL;
 		nextState(Initial);
@@ -700,47 +764,46 @@ int32_t ZrtpStateClass::evWaitConfirm2(void) {
 	/*
 	 * Confirm2:
 	 * - prepare ConfAck
-	 * - send via SRTP
-	 * - switch on to receiver security
-	 * - SecureState
+	 * - switch on security
+	 * - switch to SecureState
 	 */
 	if (first == 'c' && last == '2') {
+
+	    // send ConfAck via SRTP
+	    ZrtpPacketConfirm* cpkt = new ZrtpPacketConfirm(pkt);
 	    delete sentPacket;             // delete Confirm1 packet
 	    sentPacket = NULL;
 
-	    // send ConfAck via SRTP
-	    ZrtpPacketConfirm* cpkt = new ZrtpPacketConfirm(pkt, event->content);
-	    ZrtpPacketConf2Ack* confack = parent->prepareConf2Ack(cpkt);
+	    ZrtpPacketConf2Ack* confack = parent->prepareConf2Ack(cpkt, &errorCode);
 	    delete cpkt;
 
+            // Something went wrong during processing of the confirm2 packet
+            if (confack == NULL) {
+                sendErrorPacket(errorCode);
+                return (Done);
+            }
 	    nextState(SecureState);
 
-	    if (!parent->sendPacketSRTP(static_cast<ZrtpPacketBase *>(confack))){
+	    if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(confack))){
 		sentPacket = NULL;        // don't delete conf2Ack packet, it's preconfigured
 		nextState(Initial);
-                parent->srtpSecretsOff(ForSender);
-                parent->srtpSecretsOff(ForReceiver);
 		parent->sendInfo(Error, sendErrorTextSrtp);
 		return(Fail);
 	    }
 	    sentPacket = static_cast<ZrtpPacketBase *>(confack);
 	    parent->sendInfo(Info, "Switching to secure state");
+            // TODO: error handling here 
+            parent->srtpSecretsReady(ForSender);
+            parent->srtpSecretsReady(ForReceiver);
 
-	    return (OkDismiss);
+	    return (Done);
 	}
         return (Done);      // unknown packet for this state - Just ignore it
     }
     else {      // unknown Event type for this state
-        if (event->type == ZrtpGoClear) {
-            parent->sendInfo(Error, goClearReceived);
-        }
-        else {
-            parent->sendInfo(Error, internalProtocolError);
-        }
+        parent->sendInfo(Error, internalProtocolError);
         delete sentPacket;
         sentPacket = NULL;
-        parent->srtpSecretsOff(ForSender);
-        parent->srtpSecretsOff(ForReceiver);
         nextState(Initial);
         return (Fail);
     }
@@ -748,6 +811,7 @@ int32_t ZrtpStateClass::evWaitConfirm2(void) {
 
 /*
  * When entering this transition function
+ * - Initiator mode
  * - sentPacket contains Confirm2 packet, Confirm2 timer active
  * - sender and receiver security switched on
  */
@@ -776,13 +840,16 @@ int32_t ZrtpStateClass::evWaitConfAck(void) {
             sentPacket = NULL;
 	    parent->sendInfo(Info, "Switching to secure state");
 	    nextState(SecureState);
+      // TODO: error handling here 
+            parent->srtpSecretsReady(ForSender);
+            parent->srtpSecretsReady(ForReceiver);
 	    return (Done);
 	}
         return (Done);      // unknown packet for this state - Just ignore it
     }
     else if (event->type == Timer) {
 	if (sentPacket != NULL) {
-            if ((nextTimer(&T2) <= 0) || !parent->sendPacketSRTP(sentPacket)) {
+            if ((nextTimer(&T2) <= 0) || !parent->sendPacketZRTP(sentPacket)) {
                 parent->sendInfo(Error, resendError);
                 delete sentPacket;
                 sentPacket = NULL;
@@ -795,17 +862,113 @@ int32_t ZrtpStateClass::evWaitConfAck(void) {
         return (Done);
     }
     else {      // another Event type for this state
-        if (event->type == ZrtpGoClear) {
-            parent->sendInfo(Error, goClearReceived);
-        }
-        else {
-            parent->sendInfo(Error, internalProtocolError);
-        }
+        parent->sendInfo(Error, internalProtocolError);
         delete sentPacket;
 	sentPacket = NULL;
 	nextState(Initial);
-        parent->srtpSecretsOff(ForSender);
-        parent->srtpSecretsOff(ForReceiver);
+        return (Fail);
+    }
+}
+
+/*
+ * When entering this transition function
+ * - sentPacket contains GoClear packet, GoClear timer active
+ * The GoClear packet should no be deleted because it is a static packet.
+ */
+
+int32_t ZrtpStateClass::evWaitClearAck(void) {
+    DEBUGOUT((cout << "Checking for match in ClearAck.\n"));
+
+    char *msg, first, last;
+    uint8_t *pkt;
+
+    if (event->type == ZrtpPacket) {
+	pkt = event->data.packet;
+	msg = (char *)pkt + 4;
+
+	first = tolower(*msg);
+	last = tolower(*(msg+7));
+
+	/*
+	 * ClearAck:
+	 * - stop resending GoClear,
+	 * - switch to state AckDetected, wait for peer's Hello
+	 */
+	if (first == 'c' && last =='k') {
+	    cancelTimer();	// stop send go clear timer processing, don't delete this GoClear packet
+	    sentPacket = NULL;
+	    nextState(Initial);
+	}
+        return (Done);      // unknown packet for this state - Just ignore it
+    }
+    // Timer event triggered - this is Timer T2 to resend GoClear w/o HMAC
+    else if (event->type == Timer) {
+	if (sentPacket != NULL) {
+            if ((nextTimer(&T2)) <= 0 || !parent->sendPacketZRTP(sentPacket)) {
+                parent->sendInfo(Error, resendError);
+                sentPacket = NULL;
+                nextState(Initial);
+                return (Fail);
+            }
+        }
+        return (Done);
+    }
+    else {      // unknown Event type for this state
+        parent->sendInfo(Error, internalProtocolError);
+	sentPacket = NULL;
+	nextState(Initial);
+        return (Fail);
+    }
+}
+
+
+/*
+ * When entering this transition function
+ * - sentPacket contains Error packet, Error timer active ?? TODO
+ * The Error packet should no be deleted because it is a static packet.
+ */
+
+int32_t ZrtpStateClass::evWaitErrorAck(void) {
+    DEBUGOUT((cout << "Checking for match in ErrorAck.\n"));
+
+    char *msg, first, last;
+    uint8_t *pkt;
+
+    if (event->type == ZrtpPacket) {
+	pkt = event->data.packet;
+	msg = (char *)pkt + 4;
+
+	first = tolower(*msg);
+	last = tolower(*(msg+7));
+
+	/*
+	 * ClearAck:
+	 * - stop resending GoClear,
+	 * - switch to state AckDetected, wait for peer's Hello
+	 */
+	if (first == 'e' && last =='k') {
+	    cancelTimer();	// stop send Error timer processing, don't delete Error packet
+	    sentPacket = NULL;
+	    nextState(Initial);
+	}
+        return (Done);
+    }
+    // Timer event triggered - this is Timer T2 to resend Error.
+    else if (event->type == Timer) {
+	if (sentPacket != NULL) {
+            if ((nextTimer(&T2)) <= 0 || !parent->sendPacketZRTP(sentPacket)) {
+                parent->sendInfo(Error, resendError);
+                sentPacket = NULL;
+                nextState(Initial);
+                return (Fail);
+            }
+        }
+        return (Done);
+    }
+    else {      // unknown Event type for this state
+        parent->sendInfo(Error, internalProtocolError);
+	sentPacket = NULL;
+	nextState(Initial);
         return (Fail);
     }
 }
@@ -826,12 +989,12 @@ int32_t ZrtpStateClass::evSecureState(void) {
 
 	/*
 	 * Confirm2:
-	 * - resend ConfAck packet via SRTP
+	 * - resend ConfAck packet
 	 * - stay in state
 	 * - don't delete this confAck packet - it's preconfigured
 	 */
 	if (first == 'c' && last == '2') {
-	    if (sentPacket != NULL && !parent->sendPacketSRTP(sentPacket)) {
+	    if (sentPacket != NULL && !parent->sendPacketZRTP(sentPacket)) {
 		sentPacket = NULL;        // don't delete confAck packet, it's preconfigured
 		nextState(Initial);
                 parent->srtpSecretsOff(ForSender);
@@ -841,17 +1004,19 @@ int32_t ZrtpStateClass::evSecureState(void) {
 	    }
 	    return (Done);
 	}
-    }
-    else if (event->type == ZrtpGoClear) {
-        // TODO fix goClear handling
-        ZrtpPacketGoClear* gpkt = new ZrtpPacketGoClear(pkt);
-        ZrtpPacketClearAck* clearAck = parent->prepareClearAck(gpkt);
-        delete gpkt;
+        /*
+         * GoClear received, handle it. TODO fix go clear handling
+         */
+        if (first == 'g' && last == 'r') {
+            ZrtpPacketGoClear* gpkt = new ZrtpPacketGoClear(pkt);
+            ZrtpPacketClearAck* clearAck = parent->prepareClearAck(gpkt);
+            delete gpkt;
 
-        if (!parent->sendPacketRTP(static_cast<ZrtpPacketBase *>(clearAck))) {
-            return(Done);
-        }
+            if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(clearAck))) {
+                return(Done);
+            }
         // TODO Timeout to resend clear ack until user user confirmation
+        }
     }
     else {
 	sentPacket = NULL;
@@ -880,3 +1045,23 @@ int32_t ZrtpStateClass::nextTimer(zrtpTimer_t *t) {
     }
     return parent->activateTimer(t->time);
 }
+
+int32_t ZrtpStateClass::sendErrorPacket(uint32_t errorCode) {
+    ZrtpPacketError* err = parent->prepareError(errorCode);
+    if (!parent->sendPacketZRTP(static_cast<ZrtpPacketBase *>(err)) || (startTimer(&T2) <= 0)) {
+        nextState(Initial);
+        parent->sendInfo(Error, sendErrorText);
+        return (Fail);
+    }
+    sentPacket =  static_cast<ZrtpPacketBase *>(err);
+    nextState(WaitErrorAck);
+    return (Done);
+}
+
+/** EMACS **
+ * Local variables:
+ * mode: c++
+ * c-default-style: ellemtel
+ * c-basic-offset: 4
+ * End:
+ */
