@@ -1,10 +1,10 @@
 /*
-  Copyright (C) 2006 Werner Dittmann
+  Copyright (C) 2006-2007 Werner Dittmann
 
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _ZIDRECORD_H_
@@ -26,7 +25,7 @@
 
 typedef struct zidrecord {
     char recValid,		// if 1 record is valid, if 0: invalid
-	ownZid,			// if 1 record contains associated ZID, usually 1st record
+	ownZid,			// if 1 record contains own ZID, usually 1st record
 	rs1Valid,			// if 1 RS1 contains valid data
 	rs2Valid;			// if 1 RS2 contains valid data
     unsigned char identifier[IDENTIFIER_LEN]; // the peer's ZID
