@@ -1,10 +1,10 @@
 /*
-  Copyright (C) 2006 Werner Dittmann
+  Copyright (C) 2006-2007 Werner Dittmann
 
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _ZRTP_H_
@@ -29,6 +28,8 @@
 #include <libzrtpcpp/ZrtpPacketConf2Ack.h>
 #include <libzrtpcpp/ZrtpPacketGoClear.h>
 #include <libzrtpcpp/ZrtpPacketClearAck.h>
+#include <libzrtpcpp/ZrtpPacketError.h>
+#include <libzrtpcpp/ZrtpPacketErrorAck.h>
 #include <libzrtpcpp/ZrtpCallback.h>
 #include <libzrtpcpp/ZIDRecord.h>
 
@@ -36,6 +37,25 @@
 #define SHA256_DIGEST_LENGTH 32
 #endif
 
+
+// The ZRTP error codes accroding the specification chapter 6.9
+enum zrtpErrorCodes {
+    MalformedPacket =   0x10,
+    CriticalSWError =   0x20,
+    UnsuppZRTPVersion = 0x30,
+    HelloCompMismatch = 0x40,
+    UnsuppHashType =    0x51,
+    UnsuppCiphertype =  0x52,
+    UnsuppPKExchange =  0x53,
+    UnsuppSRTPAuthTag = 0x54,
+    UnsuppSASScheme =   0x55,
+    DHErrorWrongPV =    0x61,
+    DHErrorWrongHVI =   0x62,
+    ConfirmHMACWrong =  0x70,
+    NonceReused =       0x80,
+    EqualZIDHello =     0x90
+};
+
 class ZrtpStateClass;
 class ZrtpDH;
 
@@ -108,13 +128,10 @@ class ZRtp {
 	 * @param extHeader
 	 *    A pointer to the first byte of the extension header. Refer to
 	 *    RFC3550.
-         * @param content
-         *    Pointer to the content of the received packet. Required for
-         *    Confirm handling.
 	 * @return
 	 *    Code indicating further packet handling, see description above.
          */
-	int32_t processExtensionHeader(uint8_t *extHeader, uint8_t* content);
+	int32_t processZrtpMessage(uint8_t *extHeader);
 
         /**
 	 * Process a timeout event.
@@ -213,8 +230,6 @@ class ZRtp {
         */
        void resetSASVerified();
 
-       int Zfone;
-
  private:
      friend class ZrtpStateClass;
 
@@ -252,7 +267,7 @@ class ZRtp {
     /**
      * My computed public key
      */
-    uint8_t* pubKeyBytes;
+    uint8_t pubKeyBytes[1024];
     /**
      * Length off public key
      */
@@ -263,10 +278,14 @@ class ZRtp {
     Role myRole;
 
     /**
-     * The SAS value
+     * The human readable SAS value
      */
     std::string SAS;
 
+    /**
+     * The SAS value for signaling and alike
+     */
+    uint8_t sasValue[8];
     /**
      * The variables for the retained shared secrets
      */
@@ -291,6 +310,7 @@ class ZRtp {
      */
     uint8_t peerHvi[SHA256_DIGEST_LENGTH];
 
+    void* msgShaContext;
     /**
      * Commited Hash, Cipher, and public key algorithms
      */
@@ -306,6 +326,10 @@ class ZRtp {
      * The selected SAS type.
      */
     SupportedAuthLengths authLength;
+    /**
+     * The SHA256 hash over selected messages
+     */
+    uint8_t messageHash[SHA256_DIGEST_LENGTH];
     /**
      * The s0
      */
@@ -317,9 +341,10 @@ class ZRtp {
     uint8_t newRs1[RS_LENGTH];
 
     /**
-     * The HMAC key
+     * The GoClear HMAC keys and confirm HMAC key
      */
-    uint8_t hmacSrtp[SHA256_DIGEST_LENGTH];
+    uint8_t hmacKeyI[SHA256_DIGEST_LENGTH];
+    uint8_t hmacKeyR[SHA256_DIGEST_LENGTH];
 
     /**
      * The Initiator's srtp key and salt
@@ -334,12 +359,31 @@ class ZRtp {
     uint8_t srtpSaltR[SHA256_DIGEST_LENGTH];
 
     /**
-     * Pre-initialized packets to start off the whole game.
+     * The keys used to encrypt/decrypt the confirm message
      */
-    ZrtpPacketHello*    zrtpHello;
-    ZrtpPacketHelloAck* zrtpHelloAck;
-    ZrtpPacketConf2Ack* zrtpConf2Ack;
+    uint8_t zrtpKeyI[SHA256_DIGEST_LENGTH];
+    uint8_t zrtpKeyR[SHA256_DIGEST_LENGTH];
 
+    /**
+     * Pre-initialized packets.
+     */
+    ZrtpPacketHello    zrtpHello;
+    ZrtpPacketHelloAck zrtpHelloAck;
+    ZrtpPacketConf2Ack zrtpConf2Ack;
+    ZrtpPacketClearAck zrtpClearAck;
+    ZrtpPacketGoClear  zrtpGoClear;
+    ZrtpPacketError    zrtpError;
+    ZrtpPacketErrorAck zrtpErrorAck;
+
+    /**
+     * Holds a pre-computed DHPart2 packet. Required to compute HVI
+     */
+    ZrtpPacketDHPart* zpDH2;
+
+    /**
+     * Random IV data to encrypt the confirm data, 128 bit for AES
+     */
+    uint8_t randomIV[16];
     /**
      * Find the best Hash algorithm that was offered in Hello.
      *
@@ -418,7 +462,7 @@ class ZRtp {
     /**
      * Compute my hvi value according to ZRTP specification.
      */
-    void computeHvi(uint8_t *pv, uint32_t pvLength, ZrtpPacketHello *hello);
+    void computeHvi(ZrtpPacketDHPart* dh, ZrtpPacketHello *hello);
 
     void computeSharedSecretSet(ZIDRecord& zidRec);
 
@@ -444,7 +488,7 @@ class ZRtp {
      * @return
      *    zero if sending failed, one if packet was send
      */
-    int32_t sendPacketRTP(ZrtpPacketBase *packet);
+    int32_t sendPacketZRTP(ZrtpPacketBase *packet);
 
     /**
      * Send a ZRTP packet using SRTP.
@@ -456,9 +500,9 @@ class ZRtp {
      *    Points to the ZRTP packet.
      * @return
      *    zero if sending failed, one if packet was send
-     */
+     *
     int32_t sendPacketSRTP(ZrtpPacketBase *packet);
-
+     */
     /**
      * Activate a Timer using the host callback.
      *
@@ -486,7 +530,7 @@ class ZRtp {
      * @return
      *    A pointer to the initialized Hello packet.
      */
-    ZrtpPacketHello *prepareHello() {return zrtpHello; }
+    ZrtpPacketHello *prepareHello() {return &zrtpHello; }
 
     /**
      * Prepare a HelloAck packet.
@@ -497,12 +541,8 @@ class ZRtp {
      * @return
      *    A pointer to the initialized HelloAck packet.
      */
-    ZrtpPacketHelloAck *prepareHelloAck(ZrtpPacketHello *hello) {
-        uint8_t* cid = hello->getClientId();
-        if (*cid == 'Z') {
-            Zfone = 1;
-        }
-        return zrtpHelloAck;
+    ZrtpPacketHelloAck *prepareHelloAck() {
+        return &zrtpHelloAck;
     }
 
     /**
@@ -518,7 +558,7 @@ class ZRtp {
      * @return
      *    A pointer to the prepared Commit packet
      */
-    ZrtpPacketCommit *prepareCommit(ZrtpPacketHello *hello);
+    ZrtpPacketCommit *prepareCommit(ZrtpPacketHello *hello, uint32_t* errMsg);
 
     /**
      * Prepare the DHPart1 packet.
@@ -532,7 +572,7 @@ class ZRtp {
      * keys according to the selected cipher. Using this data we prepare our DHPart1
      * packet.
      */
-    ZrtpPacketDHPart *prepareDHPart1(ZrtpPacketCommit *commit);
+    ZrtpPacketDHPart *prepareDHPart1(ZrtpPacketCommit *commit, uint32_t* errMsg);
 
     /**
      * Prepare the DHPart2 packet.
@@ -543,7 +583,7 @@ class ZRtp {
      * Initiator.
      *
      */
-    ZrtpPacketDHPart *prepareDHPart2(ZrtpPacketDHPart *dhPart1);
+    ZrtpPacketDHPart *prepareDHPart2(ZrtpPacketDHPart* dhPart1, uint32_t* errMsg);
 
     /**
      * Prepare the Confirm1 packet.
@@ -553,7 +593,7 @@ class ZRtp {
      * as response of our DHPart1. Here we are in the role of the Responder
      *
      */
-    ZrtpPacketConfirm *prepareConfirm1(ZrtpPacketDHPart *dhPart2);
+    ZrtpPacketConfirm *prepareConfirm1(ZrtpPacketDHPart* dhPart2, uint32_t* errMsg);
 
     /**
      * Prepare the Confirm2 packet.
@@ -562,7 +602,7 @@ class ZRtp {
      * Confirm1 packet received from our peer. The peer sends the Confirm1 packet
      * as response of our DHPart2. Here we are in the role of the Initiator
      */
-    ZrtpPacketConfirm* prepareConfirm2(ZrtpPacketConfirm *confirm1);
+    ZrtpPacketConfirm* prepareConfirm2(ZrtpPacketConfirm* confirm1, uint32_t* errMsg);
 
     /**
      * Prepare the Conf2Ack packet.
@@ -571,7 +611,23 @@ class ZRtp {
      * Confirm2 packet received from our peer. The peer sends the Confirm2 packet
      * as response of our Confirm1. Here we are in the role of the Initiator
      */
-    ZrtpPacketConf2Ack* prepareConf2Ack(ZrtpPacketConfirm *confirm2);
+    ZrtpPacketConf2Ack* prepareConf2Ack(ZrtpPacketConfirm* confirm2, uint32_t* errMsg);
+
+    /**
+     * Prepare the ErrorAck packet.
+     *
+     * This method prepares the ErrorAck packet. The input to this method is the
+     * Error packet received from the peer.
+     */
+    ZrtpPacketErrorAck* prepareErrorAck(ZrtpPacketError* epkt);
+
+    /**
+     * Prepare the Error packet.
+     *
+     * This method prepares the Error packet. The input to this method is the
+     * error code to be included into the message.
+     */
+    ZrtpPacketError* prepareError(uint32_t errMsg);
 
     /**
      * Prepare a ClearAck packet.
@@ -587,6 +643,18 @@ class ZRtp {
      */
     ZrtpPacketClearAck* prepareClearAck(ZrtpPacketGoClear* gpkt);
 
+    /**
+     * Prepare a GoClearAck packet w/o HMAC
+     *
+     * Prepare a GoCLear packet without a HMAC but with a short error message.
+     * This type of GoClear is used if something went wrong during the ZRTP
+     * negotiation phase.
+     *
+     * @return
+     *     A goClear packet without HMAC
+     */
+    ZrtpPacketGoClear* prepareGoClear(uint32_t errMsg = 0);
+
     /**
      * Compare the hvi values.
      *
@@ -657,8 +725,11 @@ class ZRtp {
      *
      * @param part
      *    Defines for which part (sender or receiver) to switch on security
+     * @return
+     *    Returns false if something went wrong during initialization of SRTP
+     *    context. Propagate error back to state engine.
      */
-    void srtpSecretsReady(EnableSecurity part);
+    bool srtpSecretsReady(EnableSecurity part);
 
     /**
      * Switch off SRTP secrets.
