@@ -1,10 +1,10 @@
 /*
-  Copyright (C) 2006 Werner Dittmann
+  Copyright (C) 2006-2007 Werner Dittmann
 
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /*
@@ -53,14 +52,16 @@ class ZrtpPacketCommit : public ZrtpPacketBase {
     uint8_t* getZid()         { return commitHeader->zid; };
     uint8_t* getHvi()         { return commitHeader->hvi; };
 
-    void setHashType(uint8_t* text)    { memcpy(commitHeader->hash, text, 8); };
-    void setCipherType(uint8_t* text)  { memcpy(commitHeader->cipher, text, 8); };
-    void setAuthLen(uint8_t* text)     { memcpy(commitHeader->authlengths, text, 8); };
-    void setPubKeyType(uint8_t* text)  { memcpy(commitHeader->pubkey, text, 8); };
-    void setSasType(uint8_t* text)     { memcpy(commitHeader->sas, text, 8); };
-    void setZid(uint8_t* text)         { memcpy(commitHeader->zid, text, 12); };
-    void setHvi(uint8_t* text)         { memcpy(commitHeader->hvi, text, 32); };
+    void setHashType(uint8_t* text)    { memcpy(commitHeader->hash, text, ZRTP_WORD_SIZE); };
+    void setCipherType(uint8_t* text)  { memcpy(commitHeader->cipher, text, ZRTP_WORD_SIZE); };
+    void setAuthLen(uint8_t* text)     { memcpy(commitHeader->authlengths, text, ZRTP_WORD_SIZE); };
+    void setPubKeyType(uint8_t* text)  { memcpy(commitHeader->pubkey, text, ZRTP_WORD_SIZE); };
+    void setSasType(uint8_t* text)     { memcpy(commitHeader->sas, text, ZRTP_WORD_SIZE); };
+    void setZid(uint8_t* text)         { memcpy(commitHeader->zid, text, sizeof(commitHeader->zid)); };
+    void setHvi(uint8_t* text)         { memcpy(commitHeader->hvi, text, sizeof(commitHeader->hvi)); };
+
  private:
+     CommitPacket_t data;
 };
 
 #endif // ZRTPPACKETCOMMIT
